#! /bin/sh

VSN=1.3.0

if (test -z "$1" -o $# -ne 1) ; then
    echo " Usage: setup.sh OTP_DIR"
    echo " where: OTP_DIR   -- full path name of the Erlang/OTP"
    echo "                     installation directory"
    exit
fi

DIALYZER_OTP_DIR=$1

echo "Verifying Erlang/OTP installation..."
if (test -f $DIALYZER_OTP_DIR/lib/erlang/bin/erl) ; then
    # The user pointed out the top-level of an Erlang/OTP
    # installed using 'make install'. Silently descend to
    # the correct root directory.
    DIALYZER_OTP_DIR="$DIALYZER_OTP_DIR/lib/erlang"
fi

DIALYZER_OTP_OTP="$DIALYZER_OTP_DIR/bin/erl"
DIALYZER_OTP_ERLC="$DIALYZER_OTP_DIR/bin/erlc"
DIALYZER_HIPE_DIR="$DIALYZER_OTP_DIR/lib/hipe*"
DIALYZER_HIPE_VSN="$DIALYZER_OTP_DIR/lib/hipe*/vsn.mk"

if (test -f "$DIALYZER_OTP_OTP") ; then
    echo "  $DIALYZER_OTP_OTP exists..."
else
    echo "  Missing $DIALYZER_OTP_OTP -- check your Erlang/OTP installation"
    exit 1;
fi

if (test -f "$DIALYZER_OTP_ERLC") ; then
    echo "  $DIALYZER_OTP_ERLC exists..."
else
    echo "  Missing $DIALYZER_OTP_ERLC -- check your Erlang/OTP installation"
    exit 1;
fi

## there should NOT be any quotes in the filename of the next line
if (test -f $DIALYZER_HIPE_VSN) ; then
    ## the file contains something like HIPE_VSN = 3.4.0
    HIPE_VSN=`cat $DIALYZER_HIPE_VSN | sed s/.*=//`
    HIPE_VSN_MAJOR=`echo $HIPE_VSN | sed s/\\\\.[0-9]*\\\\.[0-9]*//`
    HIPE_VSN_MINOR=`echo $HIPE_VSN | sed s/[0-9]*\\\\.// | sed s/\\\\.[0-9]*//`
    if (test $HIPE_VSN_MAJOR -ge 3 -a $HIPE_VSN_MINOR -ge 4) ; then
      echo "  The 'hipe' application exists and is recent enough..."
    else
      echo "  The 'hipe' application of your Erlang/OTP is not recent enough"
      echo "Cannot setup Dialyzer, sorry :-("
      exit 1;
    fi
else
    ## there should NOT be any quotes in the dirname of the next line
    if (test -d $DIALYZER_HIPE_DIR) ; then
	HIPE_VSN=`basename $DIALYZER_HIPE_DIR | sed s/.*-//`
	HIPE_VSN_MAJOR=`echo $HIPE_VSN | sed s/\\\\.[0-9]*\\\\.[0-9]*//`
	HIPE_VSN_MINOR=`echo $HIPE_VSN | sed s/[0-9]*\\\\.// | sed s/\\\\.[0-9]*//`
	if (test $HIPE_VSN_MAJOR -ge 3 -a $HIPE_VSN_MINOR -ge 4) ; then
	  echo "  The installed 'hipe' application exists and is recent enough..."
	else
	  echo "  The 'hipe' application of your Erlang/OTP is not recent enough"
	  echo "Cannot setup Dialyzer, sorry :-("
	  exit 1;
	fi
    else 
	echo "  Cannot determine HiPE version -- check your Erlang/OTP installation"
	exit 1;
    fi
fi

echo "Checking dialyzer directory..."
if (test -d ebin) ; then
    echo "  The 'ebin' directory already exists..."
else
    echo "  Creating 'ebin/'"
    mkdir ebin
fi

if (test -d plt) ; then
    echo "  The 'plt'  directory already exists..."
else
    echo "  Creating 'plt/'"
    mkdir plt
fi

if (test -x "$DIALYZER_OTP_ERLC") ; then
    ## remove the file unconditionally to avoid problems when using
    ## symbolic links across filesystems in different machines.
    rm -f src/hipe_icode_type.hrl
    echo "  Creating a symbolic link to $DIALYZER_OTP_DIR/lib/hipe/icode/hipe_icode_type.hrl"
    ln -s $DIALYZER_OTP_DIR/lib/hipe*/icode/hipe_icode_type.hrl src
    ## create all files in src
    cd src
    sed -e "s;%VSN%;$VSN;" dialyzer.hrl.src > dialyzer.hrl
    for file in *.erl ; do
      echo "    Compiling $file"
      $DIALYZER_OTP_ERLC -W +debug_info -o../ebin $file
      beam_file=`basename $file .erl`".beam"
      if (test ! -f ../ebin/$beam_file); then
	  echo "Compilation of $file failed"
	  exit 1
      fi
    done
    sed -e "s;%VSN%;$VSN;" dialyzer.app.src > ../ebin/dialyzer.app
    cd ..

    ## check the .beam files for consistency
    file="utils/dialyzer_chk.erl"
    $DIALYZER_OTP_ERLC -W -o utils $file
    beam_file=`basename $file .erl`".beam"
    if (test ! -f utils/$beam_file); then
       echo "Compilation of $file failed"
       exit 1
    fi
    printf "  Checking consistency of ebin/*.beam files... "
    $DIALYZER_OTP_OTP -pa utils/ -noshell -s dialyzer_chk test `pwd` -s init stop
    rm -f utils/dialyzer_chk.beam

    ## create the dialyzer script
    sed -e "s;%OTP_DIR%;$DIALYZER_OTP_DIR;" \
	-e "s;%VSN%;$VSN;" -e "s;%DATE%;`date`;" \
	-e "s;%DIALYZER_DIR%;`pwd`;" src/dialyzer.src > dialyzer
    chmod +x dialyzer
    echo "Dialyzer is analyzing itself..."
    ./dialyzer --src --command-line "`pwd`/src"
    echo "Dialyzer setup done!"
    echo "  For the full set of Dialyzer options type: ./dialyzer --help"
else
    echo "Could not find ${DIALYZER_OTP_ERLC}"
fi
